Scriptname pchsWtHomeSlaves extends Quest  

Bool Function IsLeashedSlave(Actor akActor)
	Return Util.IsLeashedSlave(akActor)
EndFunction

Function AddSelectedSlave()
	If !Self.IsRunning()
		Self.Start()
		ToggleSlaveBeds(true)
		pchsHomeSlavesTasksQuest.Start()
	EndIf
	AddSlave(Util.SelectedPahSlave)
EndFunction

Function AddSlave(Actor akActor)
	If IsLeashedSlave(akActor)
		ReferenceAlias akAlias = GetEmptyAlias()
		If akAlias
			akAlias.ForceRefTo(akActor)
			If StorageUtil.GetIntValue(akActor, "pchsHomeSlaveTrainingFactRank", Missing = -1) >= 0
				akActor.SetFactionRank(pchsHomeSlaveTrainingFact, StorageUtil.GetIntValue(akActor, "pchsHomeSlaveTrainingFactRank"))
			Else
				akActor.SetFactionRank(pchsHomeSlaveTrainingFact, 0)
			EndIf
			Fav.ModStat("Father is pleased to have another slut in his harem", Favour = GetAssignValue(akActor), Depravity = 0.0, Submission = 0.0, Sender = Self)
			StorageUtil.SetFloatValue(akActor, "pchsWtHomeSlaveAssignTime", Utility.GetCurrentGameTime())
			StorageUtil.FormListAdd(None, "pchsWtHomeSlavesList", akActor, AllowDuplicate = false)
			pchsHomeSlaveCount.SetValue(StorageUtil.FormListCount(None, "pchsWtHomeSlavesList"))
			akActor.AddToFaction(Game.GetFormFromFile(0x00DFE6, "paradise_halls.esm") as Faction) ; PAHCaged
			akActor.EvaluatePackage()
			Debug.Notification(akActor.GetDisplayName() + " added to home servitude")
		Else
			Debug.Notification("No empty home slave slots")
		EndIf
	
	Else
		Debug.Messagebox(akActor.GetDisplayName() + " needs to have a leash collar equipped before she can be assigned as a home slave")
	EndIf
EndFunction

Function RemoveSelectedSlave()
	RemoveSlave(Util.SelectedPahSlave)
EndFunction

Function RemoveSlave(Actor akActor)
	ReferenceAlias akAlias = GetSlaveAlias(akActor)
	If akAlias
		UnassignTask(akActor)
		StorageUtil.SetIntValue(akActor, "pchsHomeSlaveTrainingFactRank", akActor.GetFactionRank(pchsHomeSlaveTrainingFact))
		akActor.RemoveFromFaction(pchsHomeSlaveTrainingFact)
		StorageUtil.SetFloatValue(akActor, "pchsWtHomeSlaveUnAssignTime", Utility.GetCurrentGameTime())
		StorageUtil.FormListRemove(None, "pchsWtHomeSlavesList", akActor)
		pchsHomeSlaveCount.SetValue(StorageUtil.FormListCount(None, "pchsWtHomeSlavesList"))
		akAlias.Clear()
		akActor.EvaluatePackage()
		Debug.Notification(akActor.GetDisplayName() + " removed from home servitude")
	Else
		Debug.Trace("pchs: RemoveSlave(): Actor not found: " + akActor)
	EndIf
EndFunction

Function RemoveAllSlaves()
	Int i = 0
	Actor Slave
	While i < StorageUtil.FormListCount(None, "pchsWtHomeSlavesList")
		Slave = StorageUtil.FormListGet(None, "pchsWtHomeSlavesList", i) as Actor
		RemoveSlave(Slave)
		i += 1
	EndWhile
EndFunction

ReferenceAlias Function GetEmptyAlias()
	Int i = 0
	While i < Self.GetNumAliases()
		If !(Self.GetNthAlias(i) as ReferenceAlias).GetReference()
			Return Self.GetNthAlias(i) as ReferenceAlias
		EndIf
		i += 1
	EndWhile
	Return None
EndFunction

ReferenceAlias Function GetSlaveAlias(Actor Slave)
	Int i = 0
	While i < Self.GetNumAliases()
			If (Self.GetNthAlias(i) as ReferenceAlias).GetReference() as Actor == Slave
				Return Self.GetNthAlias(i) as ReferenceAlias
			EndIf
		i += 1
	EndWhile
	Return None
EndFunction

Actor Function GetRandomHomeSlave()
	If StorageUtil.FormListCount(None, "pchsWtHomeSlavesList") > 0
		Return StorageUtil.FormListGet(None, "pchsWtHomeSlavesList", Utility.RandomInt(0, StorageUtil.FormListCount(None, "pchsWtHomeSlavesList") - 1)) as Actor
	EndIf
	Return None
EndFunction

Actor Function GetSlaveAssignedToTask(String Task)
	Return StorageUtil.FormListGet(None, "pchsWtHomeSlaves_" + Task, Utility.RandomInt(0, StorageUtil.FormListCount(None, "pchsWtHomeSlaves_" + Task) - 1)) as Actor
EndFunction

Float Function GetPahSubmission(Actor akActor)
	Return akActor.GetFactionRank(Util.PAHSubmission) as Float
EndFunction

Float Function GetHomeTraining(Actor akActor)
	Float HomeTraining = akActor.GetFactionRank(pchsHomeSlaveTrainingFact) as Float
	If HomeTraining < 0.0
		HomeTraining = StorageUtil.GetIntValue(akActor, "pchsHomeSlaveTrainingFactRank", Missing = 0) as Float
	EndIf
	Return HomeTraining
EndFunction

Float Function GetSexTraining(Actor akActor)
	Return akActor.GetFactionRank(Util.PAHTrainSex) as Float
EndFunction

Bool Function PassHomeTask(Actor akActor)
	If !akActor
		akActor = GetRandomHomeSlave()
	EndIf
	If akActor
						; Base + weight
		Float PassChance = 0.1 + (0.44 * (GetHomeTraining(akActor) / 100.0)) + (0.44 * (GetPahSubmission(akActor) / 100.0))
		Return PassChance >= Utility.RandomFloat(0.0, 1.0)
	EndIf
	Return false
EndFunction

Bool Function PassHomeSexTask(Actor akActor)
	If !akActor
		akActor = GetRandomHomeSlave()
	EndIf
	If akActor
						; Base + weight
		Float PassChance = 0.1 + (0.29 * (GetSexTraining(akActor) / 100.0)) + (0.29 * (GetPahSubmission(akActor) / 100.0)) + (0.29 * (akActor.GetLeveledActorBase().GetWeight() / 100.0))
		Return PassChance >= Utility.RandomFloat(0.0, 1.0)
	EndIf
	Return false
EndFunction

Function TaskSuccess(Actor Slave, Int TaskType)
	; TaskType: 0 - Normal task, 1 - Sex task
	
	Slave.SetFactionRank(pchsHomeSlaveTrainingFact, PapyrusUtil.ClampInt(Slave.GetFactionRank(pchsHomeSlaveTrainingFact) + 1, 0, 100))
EndFunction

Function TaskFail(Actor Slave, Int TaskType)
	; TaskType: 0 - Normal task, 1 - Sex task
	
	StorageUtil.AdjustIntValue(Slave, "pchsWtSlaveFailedTaskCount", 1)
EndFunction

Function WhipSuccess(Actor Slave)
	StorageUtil.AdjustIntValue(Slave, "pchsWtSlaveFailedTaskCount", -1)
	Slave.SetFactionRank(pchsHomeSlaveTrainingFact, PapyrusUtil.ClampInt(Slave.GetFactionRank(pchsHomeSlaveTrainingFact) + 1, 0, 100))
	Debug.Notification(Slave.GetDisplayName() + " learned 1 point of home training")
	If 15.0 > Utility.RandomFloat(0.0, 100.0)
		Fav.ModStat("Father is pleased to see me discipline his toy", Favour = 1.0, Depravity = 0.0, Submission = 0.0, Sender = Self)
	EndIf
EndFunction

Float Function GetFavourValue(Actor akActor)
	Float Weight = (akActor.GetLeveledActorBase().GetWeight() / 100.0) * 5.0
	Float HouseTraining = (GetHomeTraining(akActor) / 100.0) * 5.0
	Float SexTraining = (GetSexTraining(akActor) / 100.0) * 5.0
	Float Submission = (GetPahSubmission(akActor) / 100.0) * 5.0
	Return ((Weight + HouseTraining + SexTraining + Submission) as Int) as Float
EndFunction

Function DisplayUnassignCost()
	If Util.SelectedPahSlave
		Int Cost = GetUnassignCost(Util.SelectedPahSlave) as Int
		pchsHomeSlaveUnassignCost.SetValue(Cost)
		Debug.Messagebox("Cost to unassign: " + Cost + " Favour")
	Else
		Debug.Notification("No slave selected")
	EndIf
EndFunction

Float Function GetAssignValue(Actor akActor)
	GetFavourValue(akActor)
	Float DaysToUnBoredom = 2.0
	Float Value = GetFavourValue(akActor)
	Float TimeMod = 1.0
	If Utility.GetCurrentGameTime() <= StorageUtil.GetFloatValue(akActor, "pchsWtHomeSlaveUnAssignTime", Missing = -999.0) + DaysToUnBoredom
		TimeMod = 1.0 - PapyrusUtil.ClampFloat(((StorageUtil.GetFloatValue(akActor, "pchsWtHomeSlaveAssignTime") + DaysToUnBoredom) - Utility.GetCurrentGameTime()) / DaysToUnBoredom, 0.0, 1.0)
	EndIf
	
	;Debug.Messagebox("TimeMod: " + TimeMod)
	;Debug.Messagebox("Favour Gain: " + PapyrusUtil.ClampInt((Value * TimeMod) as Int, 0, 999))
	Return PapyrusUtil.ClampInt((Value * TimeMod) as Int, 0, 999)
EndFunction

Float Function GetUnassignCost(Actor akActor)
	Float DaysToBoredom = 2.0
	Float Value = GetFavourValue(akActor)
	Float TimeMod = PapyrusUtil.ClampFloat(((StorageUtil.GetFloatValue(akActor, "pchsWtHomeSlaveAssignTime") + DaysToBoredom) - Utility.GetCurrentGameTime()) / DaysToBoredom, 0.0, 1.0)
	;Debug.Messagebox("TimeMod: " + TimeMod)
	Return Math.Ceiling(Value * TimeMod)
EndFunction

Function AssignTask(Actor akActor, String Task)
	String TaskReadable
	If Task == "FatherFood"
		pchsHomeSlaveTaskFatherFood.SetValue(1.0)
		TaskReadable = "Food & Drink"
	ElseIf Task == "Clean"
		pchsHomeSlaveTaskClean.SetValue(1.0)
		TaskReadable = "Cleaning"
	ElseIf Task == "SupplyRun"
		pchsHomeSlaveTaskSupplyRun.SetValue(1.0)
		TaskReadable = "Supply Run"
	ElseIf Task == "FatherSex"
		pchsHomeSlaveTaskFatherSex.SetValue(1.0)
		TaskReadable = "Father Sex"
	ElseIf Task == "AldoSex"
		pchsHomeSlaveTaskAldoSex.SetValue(1.0)
		TaskReadable = "Aldo Sex"
	ElseIf Task == "Brutus"
		pchsHomeSlaveTaskBrutus.SetValue(1.0)
		TaskReadable = "Brutus"
	ElseIf Task == "Firewood"
		pchsHomeSlaveTaskFirewood.SetValue(1.0)
		TaskReadable = "Firewood"
	EndIf
	StorageUtil.FormListAdd(None, "pchsWtHomeSlaves_" + Task, akActor, AllowDuplicate = false)
	AssignTaskAlias(Task, akActor)
	akActor.AddToFaction(pchsHomeSlaveTaskAssignedFact)
	Debug.Notification(akActor.GetDisplayName() + " assigned to task: " + TaskReadable)
EndFunction

Function UnassignTask(Actor akActor)
	String Task
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_FatherFood", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_FatherFood", akActor)
		pchsHomeSlaveTaskFatherFood.SetValue(0.0)
		Task = "Food & Drink"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Clean", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_Clean", akActor)
		pchsHomeSlaveTaskClean.SetValue(0.0)
		Task = "Cleaning"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_SupplyRun", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_SupplyRun", akActor)
		pchsHomeSlaveTaskSupplyRun.SetValue(0.0)
		Task = "Supply Run"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_FatherSex", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_FatherSex", akActor)
		pchsHomeSlaveTaskFatherSex.SetValue(0.0)
		Task = "Father Sex"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_AldoSex", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_AldoSex", akActor)
		pchsHomeSlaveTaskAldoSex.SetValue(0.0)
		Task = "Aldo Sex"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Brutus", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_Brutus", akActor)
		pchsHomeSlaveTaskBrutus.SetValue(0.0)
		Task = "Brutus"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Firewood", akActor) > -1
		StorageUtil.FormListRemove(None, "pchsWtHomeSlaves_Firewood", akActor)
		pchsHomeSlaveTaskFirewood.SetValue(0.0)
		Task = "Firewood"
	EndIf
	
	UnAssignTaskAlias(akActor)
	akActor.RemoveFromFaction(pchsHomeSlaveTaskAssignedFact)
	Debug.Notification(akActor.GetDisplayName() + " unassigned from task: " + Task)
EndFunction

String Function GetTaskDisplayString(Actor akActor)
	String Task = "None"
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_FatherFood", akActor) > -1
		Task = "Food & Drink"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Clean", akActor) > -1
		Task = "Cleaning"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_SupplyRun", akActor) > -1
		Task = "Supply Run"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_FatherSex", akActor) > -1
		Task = "Father Sex"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_AldoSex", akActor) > -1
		Task = "Aldo Sex"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Brutus", akActor) > -1
		Task = "Brutus"
	EndIf
	If StorageUtil.FormListFind(None, "pchsWtHomeSlaves_Firewood", akActor) > -1
		Task = "Firewood"
	EndIf
	Return Task
EndFunction

String Function GetTaskPriorityString(Actor akActor)
	If akActor.IsInFaction(pchsHomeSlaveTaskImmediateFact)
		Return "High"
	EndIf
	Return "Low"
EndFunction

Function ToggleTaskImmediate(Actor akActor)
	If akActor.IsInFaction(pchsHomeSlaveTaskImmediateFact)
		akActor.RemoveFromFaction(pchsHomeSlaveTaskImmediateFact)
	Else
		akActor.AddToFaction(pchsHomeSlaveTaskImmediateFact)
	EndIf
EndFunction

Int Function GetTaskAlias(String Task)
	If Task == "FatherFood"
		Return 0
	ElseIf Task == "Clean"
		Return 1
	ElseIf Task == "SupplyRun"
		Return 2
	ElseIf Task == "FatherSex"
		Return 3
	ElseIf Task == "AldoSex"
		Return 4
	ElseIf Task == "Brutus"
		Return 5
	ElseIf Task == "Firewood"
		Return 6
	EndIf
	Return -1
EndFunction

Function AssignTaskAlias(String Task, Actor akActor)
	Int TaskIndex = GetTaskAlias(Task)
	If TaskIndex > -1
		(pchsHomeSlavesTasksQuest.GetNthAlias(TaskIndex) as ReferenceAlias).ForceRefTo(akActor)
		akActor.EvaluatePackage()
	Else
		Debug.Trace("pchs: AssignTaskAlias(): Task alias not found for task: " + Task)
	EndIf
EndFunction

Function UnAssignTaskAlias(Actor akActor)
	Int i = 0
	While i < pchsHomeSlavesTasksQuest.GetNumAliases()
		If (pchsHomeSlavesTasksQuest.GetNthAlias(i) as ReferenceAlias).GetReference() as Actor == akActor
			(pchsHomeSlavesTasksQuest.GetNthAlias(i) as ReferenceAlias).Clear()
		EndIf
		i += 1
	EndWhile
	akActor.EvaluatePackage()
EndFunction

Function ToggleSlaveBeds(Bool EnableThem)
	ObjectReference Bed1 = Game.GetFormFromFile(0x3F5DED, "pchsWartimes.esp") as ObjectReference
	ObjectReference Bed2 = Game.GetFormFromFile(0x3F5DEC, "pchsWartimes.esp") as ObjectReference
	If EnableThem
		Bed1.Enable()
		Bed2.Enable()
	Else
		Bed1.Disable()
		Bed2.Disable()
	EndIf
EndFunction

Function HomeLoaded()

EndFunction

Form[] Property HomeSlaves Auto Hidden

Quest Property pchsZazFurnScan Auto
Quest Property pchsHomeSlavesTasksQuest Auto

Faction Property pchsHomeSlaveTrainingFact Auto
Faction Property pchsHomeSlaveTaskAssignedFact Auto
Faction Property pchsHomeSlaveTaskImmediateFact Auto

GlobalVariable Property pchsHomeSlaveCount Auto
GlobalVariable Property pchsHomeSlaveUnassignCost Auto

GlobalVariable Property pchsHomeSlaveTaskFatherFood Auto
GlobalVariable Property pchsHomeSlaveTaskClean Auto
GlobalVariable Property pchsHomeSlaveTaskAldoSex Auto
GlobalVariable Property pchsHomeSlaveTaskBrutus Auto
GlobalVariable Property pchsHomeSlaveTaskSupplyRun Auto
GlobalVariable Property pchsHomeSlaveTaskFatherSex Auto
GlobalVariable Property pchsHomeSlaveTaskFirewood Auto

GlobalVariable Property GameHour Auto

pchsWtUtil Property Util Auto
pchsWtFavour Property Fav Auto
